/**
 * 
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.codehaus.activemq.service.impl;

import EDU.oswego.cs.dl.util.concurrent.ConcurrentHashMap;
import org.codehaus.activemq.DuplicateDurableSubscriptionException;
import org.codehaus.activemq.broker.BrokerClient;
import org.codehaus.activemq.filter.Filter;
import org.codehaus.activemq.message.ConsumerInfo;
import org.codehaus.activemq.service.Dispatcher;
import org.codehaus.activemq.service.Subscription;
import org.codehaus.activemq.service.SubscriptionContainer;

import javax.jms.JMSException;
import java.util.Iterator;
import java.util.Map;

/**
 * A default RAM only implementation of the {@link SubscriptionContainer}
 *
 * @version $Revision: 1.1 $
 */
public class SubscriptionContainerImpl implements SubscriptionContainer {
    private Map subscriptions;

    public SubscriptionContainerImpl() {
        this(new ConcurrentHashMap());
    }

    public SubscriptionContainerImpl(Map subscriptions) {
        this.subscriptions = subscriptions;
    }

    public String toString() {
        return super.toString() + "[size:" + subscriptions.size() + "]";
    }

    public Subscription getSubscription(String consumerId) {
        return (Subscription) subscriptions.get(consumerId);
    }

    public void addSubscription(String consumerId, Subscription subscription) {
    }

    public Subscription removeSubscription(String consumerId) {
        return (Subscription) subscriptions.remove(consumerId);
    }

    public Iterator subscriptionIterator() {
        return subscriptions.values().iterator();
    }

    public Subscription makeSubscription(Dispatcher dispatcher, ConsumerInfo info, Filter filter) {
        Subscription subscription = createSubscription(dispatcher, info, filter);
        subscriptions.put(info.getConsumerId(), subscription);
        return subscription;
    }

    public void checkForDuplicateDurableSubscription(BrokerClient client, ConsumerInfo info) throws JMSException {
        for (Iterator iter = subscriptions.values().iterator(); iter.hasNext();) {
            Subscription subscription = (Subscription) iter.next();
            if (subscription.isSameDurableSubscription(info)) {
                throw new DuplicateDurableSubscriptionException(info);
            }
        }
    }

    protected Subscription createSubscription(Dispatcher dispatcher, ConsumerInfo info, Filter filter) {
        return new SubscriptionImpl(dispatcher, info, filter);
    }
}
